# Generated by roxygen2: do not edit by hand

export(architect_release)
export(architect_vignette)
export(audit_dependencies)
export(audit_script)
export(benchmark_branches)
export(bootstrap_dev_env)
export(detect_masking)
export(dictate_dictionary)
export(dispatch_checkpoints)
export(export_snapshot)
export(hunt_zombies)
export(loop_guardian)
export(manage_deprecation)
export(mask_identity)
export(network_diplomat)
export(remove_package)
export(remove_user_installed_packages)
export(scaffold_parallel)
export(scaffold_tests)
export(scan_dependencies)
export(setup_preflight)
export(setup_sentinel)
export(simulate_clean_room)
export(sweep_memory)
export(sweep_temp_cache)
importFrom(grDevices,dev.list)
importFrom(grDevices,graphics.off)
importFrom(graphics,par)
importFrom(stats,setNames)
importFrom(tools,package_dependencies)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,remove.packages)
importFrom(utils,select.list)
importFrom(utils,tail)
